/**         Graphique.h        **
*
*  Moteur graphique du jeu
*
**/

#ifndef GRAPHIQUE
#define GRAPHIQUE

#include <iostream>
#include <vector>
#include <SDL.h>
#include <SDL_opengl.h>
#include <time.h>

#include "../Particule.h"

#include "../divers.h"
#include "../define.h"

#include "../Objet.h"

class Graphique
{
public:
    Graphique();
    ~Graphique();
    static bool chargerTextures();


    // Dans Graphique_Background.cpp
    static void dessinerBackground();
    static void changerBackground(int fondDemander);

    // Dans Graphique_Objet.cpp
    static void dessinerDecors(std::vector <Objet> &objets, float joueurX, float joueurY);
    static void getInfos(int id, double &x, double &y, double &w, double &h, int &largeur, int &hauteur);

    // Dans Graphique_Pieces.cpp.
    static void animerPieces(std::vector <Objet> &objets, int id, double &x, double &y, double &w, double &h, int &largeur, int &hauteur);

    // Dans Graphique_Goomba.cpp
    static void animerGoomba(std::vector <Objet> &objets, int id, double &x, double &y, double &w, double &h, int &largeur, int &hauteur);

    // Dans Graphique_Joueur.cpp
    static void dessinerJoueur(int &action, int etat, int direction, float joueurX, float joueurY, float vx, float vy);
    static void gererAnimationJoueur(int &animationJoueur, int etat, int &action, int direction, float vx, float vy);
    static void getInfosJoueur(int &animationJoueur, int action, double &x, double &y, double &w, double &h, int &largeur, int &hauteur);
    static void afficherVies(int nombreVies, int degats);
    static void afficherDegats(int degats);
    static void getInfoDegats(int chiffre, double &x, double &y, double &w, double &h, int &largeur, int &hauteur);
    static void afficherSolRessucite(float joueurX, float joueurY);

    // Dans particules
    static void Graphique::dessinerParticules(std::vector<Particule> &particules, int tempsEcoule);
    static void Graphique::initParticules(std::vector<Particule> &particules, float joueurX, float joueurY);

private:
    // Background
    static GLuint m_IMG_background;

    // Textures
    static GLuint m_IMG_blocs;
    static GLuint m_IMG_sols;
    static GLuint m_IMG_tuyaux;
    static GLuint m_IMG_bonus;
    static GLuint m_IMG_decors;
    static GLuint m_IMG_monstres;
    static GLuint m_IMG_pieces;

    // Goomba
    static GLuint m_IMG_goomba;

    // Joueur
    static GLuint m_IMG_joueur;
    static GLuint m_IMG_degats;
};

#endif
