#include "Graphique.h"
#include "../Moteur.h"

extern Moteur moteur;

// Joueur
GLuint Graphique::m_IMG_joueur           = 0;
GLuint Graphique::m_IMG_degats           = 0;

void Graphique::dessinerJoueur(int &action, int etat, int direction, float joueurX, float joueurY, float vx, float vy)
{
    // Gerer l'animation du joueur ( Pour animer comme un .gif )
    static int animationJoueur = 0;

    // Les infos sur la partie de l'image a recuperer
    double x = 0;
    double y = 0.964;
    double w = 0.023;
    double h = 0.036;

    // Les infos sur la taille et la hauteur du personnage.
    int hauteur = 36;
    int largeur = 24;

    // Savoir si l'on affiche le joueur au centre ou a sa vraie position.
    float afficherJoueurX = (moteur.getLargeurEcran() / 2);
    if(joueurX < (moteur.getLargeurEcran() / 2))
        afficherJoueurX = joueurX;

    // Si le joueur arrive au bout du niveau
    if(joueurX > LARGEUR_MAX_CARTE - (moteur.getLargeurEcran() / 2))
        afficherJoueurX = joueurX - LARGEUR_MAX_CARTE + moteur.getLargeurEcran();

    // Si le joueur est en bas du niveau
    float afficherJoueurY = (moteur.getHauteurEcran() / 2);
    if(joueurY < (moteur.getHauteurEcran() / 2))
        afficherJoueurY = joueurY;

    // Si le joueur est en haut du niveau
    if(joueurY > HAUTEUR_MAX_CARTE - (moteur.getHauteurEcran() / 2))
        afficherJoueurY = joueurY - HAUTEUR_MAX_CARTE + moteur.getHauteurEcran();

    // Cas ou l'on ressucite
    if(etat == RESSUCITE)
    {
        Graphique::afficherSolRessucite(afficherJoueurX, afficherJoueurY);
        action = ATTEND_DROITE;
    }

    // Animer le joueur eb fonction du temps.
    gererAnimationJoueur(animationJoueur, etat, action, direction, vx, vy);

    // Rcupration des infos en fonction de l'image a afficher
    getInfosJoueur(animationJoueur, action, x, y, w, h, largeur, hauteur);

    glEnable(GL_TEXTURE_2D);
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

        glBindTexture(GL_TEXTURE_2D, m_IMG_joueur);
        glBegin(GL_TRIANGLE_STRIP);
            glTexCoord2f(    x,     y);            glVertex2d(afficherJoueurX, afficherJoueurY);
            glTexCoord2f(    x, y + h);            glVertex2d(afficherJoueurX , afficherJoueurY + hauteur);
            glTexCoord2f(x + w,     y);            glVertex2d(afficherJoueurX + largeur, afficherJoueurY);
            glTexCoord2f(x + w, y + h);            glVertex2d(afficherJoueurX + largeur, afficherJoueurY + hauteur);
        glEnd();
    glColor4ub(255,255,255, 255);
    glDisable(GL_BLEND);
    glDisable(GL_TEXTURE_2D);
}

void Graphique::gererAnimationJoueur(int &animationJoueur, int etat, int &action, int direction, float vx, float vy)
{
    static int premiereCollisionSol = 0;
    static int lastTime             = 0;

    /* --------------------------
        Choix de l'animation a faire
       -------------------------- */
    if(vy == 0 || etat == RESSUCITE)
    {
        if(vx >= -0.2 && vx <= 0.2 || etat == RESSUCITE)
        {
             if(direction == GAUCHE)
                action = ATTEND_GAUCHE;
             else
                action = ATTEND_DROITE;
        }
    }

    if(vy > 0 && etat != RESSUCITE)
    {
        if(direction == GAUCHE)
            action = DESCEND_GAUCHE;
        else
            action = DESCEND_DROITE;
    }

    if(vy < -0.2)
    {
        if(direction == GAUCHE)
            action = SAUTE_GAUCHE;
        else
            action = SAUTE_DROITE;
    }

    // Tombe de haut l'atterissage se fera acroupis :)
    if(vy > 1.5)
        premiereCollisionSol =1;

    if(vy == 0 && premiereCollisionSol > 0)
    {
        if(direction == GAUCHE)
            action = COLLISION_SOL_GAUCHE;
        else
            action = COLLISION_SOL_DROITE;

        premiereCollisionSol++;
        if(premiereCollisionSol > 8)
            premiereCollisionSol = 0;
    }


    /* --------------------------
          Animation du sprite
       -------------------------- */
    // Si lem ode bulletTime est off.
    if(moteur.getEtatSpecial() == false)
    {
        if(action == COURS_GAUCHE || action == COURS_DROITE)
        {
            if(SDL_GetTicks() - lastTime > VITESSE_ANIMATION_MARIO_COURT)
            {
                animationJoueur++;
                if(animationJoueur > 11)
                    animationJoueur = 0;

                lastTime = SDL_GetTicks();
            }
        }
        if(action == ATTEND_GAUCHE || action == ATTEND_DROITE)
        {
            if(SDL_GetTicks() - lastTime > VITESSE_ANIMATION_MARIO_ATTEND)
            {
                animationJoueur++;
                if(animationJoueur > 7)
                    animationJoueur = 0;

                lastTime = SDL_GetTicks();
            }
        }
    }
    else
    {
        if(action == COURS_GAUCHE || action == COURS_DROITE)
        {
            if(SDL_GetTicks() - lastTime > VITESSE_ANIMATION_MARIO_COURT + 150)
            {
                animationJoueur++;
                if(animationJoueur > 11)
                    animationJoueur = 0;

                lastTime = SDL_GetTicks();
            }
        }
        if(action == ATTEND_GAUCHE || action == ATTEND_DROITE)
        {
            if(SDL_GetTicks() - lastTime > VITESSE_ANIMATION_MARIO_ATTEND + 150)
            {
                animationJoueur++;
                if(animationJoueur > 7)
                    animationJoueur = 0;

                lastTime = SDL_GetTicks();
            }
        }
    }
}

void Graphique::getInfosJoueur(int &animationJoueur, int action, double &x, double &y, double &w, double &h, int &largeur, int &hauteur)
{
    switch(action)
    {
        case COURS_DROITE:
            switch(animationJoueur)
            {
                /* ------------- */
                /* Attend droite */
                /* ------------- */
                case 0:
                    x = 0;
                    y = 0.895;
                    w = 0.029;
                    h = 0.028;
                    hauteur = 29;
                    largeur = 30;
                    break;
                case 1:
                    x = 0.029;
                    y = 0.895;
                    w = 0.028;
                    h = 0.030;
                    hauteur = 31;
                    largeur = 29;
                    break;
                case 2:
                    x = 0.058;
                    y = 0.895;
                    w = 0.024;
                    h = 0.030;
                    hauteur = 31;
                    largeur = 25;
                    break;
                case 3:
                    x = 0.082;
                    y = 0.895;
                    w = 0.020;
                    h = 0.031;
                    hauteur = 32;
                    largeur = 21;
                    break;
                case 4:
                    x = 0.103;
                    y = 0.895;
                    w = 0.026;
                    h = 0.028;
                    hauteur = 29;
                    largeur = 27;
                    break;
                case 5:
                    x = 0.129;
                    y = 0.895;
                    w = 0.029;
                    h = 0.028;
                    hauteur = 29;
                    largeur = 30;
                    break;
                case 6:
                    x = 0.158;
                    y = 0.895;
                    w = 0.031;
                    h = 0.030;
                    hauteur = 31;
                    largeur = 32;
                    break;
                case 7:
                    x = 0.190;
                    y = 0.895;
                    w = 0.029;
                    h = 0.029;
                    hauteur = 30;
                    largeur = 30;
                    break;
                case 8:
                    x = 0.219;
                    y = 0.895;
                    w = 0.026;
                    h = 0.029;
                    hauteur = 30;
                    largeur = 27;
                    break;
                case 9:
                    x = 0.245;
                    y = 0.895;
                    w = 0.020;
                    h = 0.031;
                    hauteur = 32;
                    largeur = 21;
                    break;
                case 10:
                    x = 0.266;
                    y = 0.895;
                    w = 0.024;
                    h = 0.030;
                    hauteur = 31;
                    largeur = 25;
                    break;
                case 11:
                    x = 0.290;
                    y = 0.895;
                    w = 0.028;
                    h = 0.030;
                    hauteur = 31;
                    largeur = 29;
                    break;
                default:
                    break;
            }
            break;
        case COURS_GAUCHE:
            switch(animationJoueur)
            {
                /* ------------- */
                /* Attend droite */
                /* ------------- */
                case 0:
                    x = 0;
                    y = 0.862;
                    w = 0.029;
                    h = 0.028;
                    hauteur = 29;
                    largeur = 30;
                    break;
                case 1:
                    x = 0.029;
                    y = 0.862;
                    w = 0.028;
                    h = 0.030;
                    hauteur = 31;
                    largeur = 29;
                    break;
                case 2:
                    x = 0.058;
                    y = 0.862;
                    w = 0.024;
                    h = 0.030;
                    hauteur = 31;
                    largeur = 25;
                    break;
                case 3:
                    x = 0.082;
                    y = 0.862;
                    w = 0.020;
                    h = 0.031;
                    hauteur = 32;
                    largeur = 21;
                    break;
                case 4:
                    x = 0.103;
                    y = 0.862;
                    w = 0.026;
                    h = 0.028;
                    hauteur = 29;
                    largeur = 27;
                    break;
                case 5:
                    x = 0.129;
                    y = 0.862;
                    w = 0.029;
                    h = 0.028;
                    hauteur = 29;
                    largeur = 30;
                    break;
                case 6:
                    x = 0.158;
                    y = 0.862;
                    w = 0.031;
                    h = 0.030;
                    hauteur = 31;
                    largeur = 32;
                    break;
                case 7:
                    x = 0.190;
                    y = 0.862;
                    w = 0.029;
                    h = 0.029;
                    hauteur = 30;
                    largeur = 30;
                    break;
                case 8:
                    x = 0.219;
                    y = 0.862;
                    w = 0.026;
                    h = 0.029;
                    hauteur = 30;
                    largeur = 27;
                    break;
                case 9:
                    x = 0.245;
                    y = 0.862;
                    w = 0.020;
                    h = 0.031;
                    hauteur = 32;
                    largeur = 21;
                    break;
                case 10:
                    x = 0.266;
                    y = 0.862;
                    w = 0.024;
                    h = 0.030;
                    hauteur = 31;
                    largeur = 25;
                    break;
                case 11:
                    x = 0.290;
                    y = 0.862;
                    w = 0.028;
                    h = 0.030;
                    hauteur = 31;
                    largeur = 29;
                    break;
                default:
                    break;
            }
            break;
        case ATTEND_DROITE:
            switch(animationJoueur)
            {
                /* ------------- */
                /* Attend droite */
                /* ------------- */
                case 0:
                    x = 0;
                    y = 0.964;
                    w = 0.024;
                    h = 0.034;
                    hauteur = 36;
                    largeur = 24;
                    break;
                case 1:
                    x = 0.025;
                    y = 0.964;
                    w = 0.023;
                    h = 0.035;
                    hauteur = 37;
                    largeur = 24;
                    break;
                case 2:
                    x = 0.048;
                    y = 0.964;
                    w = 0.023;
                    h = 0.036;
                    hauteur = 38;
                    largeur = 24;
                    break;
                case 3:
                    x = 0.071;
                    y = 0.964;
                    w = 0.023;
                    h = 0.035;
                    hauteur = 38;
                    largeur = 23;
                    break;
                case 4:
                    x = 0.094;
                    y = 0.964;
                    w = 0.021;
                    h = 0.034;
                    hauteur = 37;
                    largeur = 22;
                    break;
                // Sens inverse maintenant
                case 5:
                    x = 0.071;
                    y = 0.964;
                    w = 0.023;
                    h = 0.035;
                    hauteur = 38;
                    largeur = 23;
                    break;
                case 6:
                    x = 0.048;
                    y = 0.964;
                    w = 0.023;
                    h = 0.036;
                    hauteur = 38;
                    largeur = 24;
                    break;
                case 7:
                    x = 0.025;
                    y = 0.964;
                    w = 0.023;
                    h = 0.035;
                    hauteur = 37;
                    largeur = 24;
                    break;
                default:
                    break;
            }
            break;
        case ATTEND_GAUCHE:
            switch(animationJoueur)
            {
                /* ------------- */
                /* Attend gauche */
                /* ------------- */
                case 0:
                    x = 0;
                    y = 0.927;
                    w = 0.024;
                    h = 0.034;
                    hauteur = 36;
                    largeur = 24;
                    break;
                case 1:
                    x = 0.025;
                    y = 0.927;
                    w = 0.023;
                    h = 0.035;
                    hauteur = 37;
                    largeur = 24;
                    break;
                case 2:
                    x = 0.048;
                    y = 0.927;
                    w = 0.023;
                    h = 0.036;
                    hauteur = 38;
                    largeur = 24;
                    break;
                case 3:
                    x = 0.071;
                    y = 0.927;
                    w = 0.023;
                    h = 0.035;
                    hauteur = 38;
                    largeur = 23;
                    break;
                case 4:
                    x = 0.094;
                    y = 0.927;
                    w = 0.021;
                    h = 0.034;
                    hauteur = 37;
                    largeur = 22;
                    break;
                // Sens inverse maintenant
                case 5:
                    x = 0.071;
                    y = 0.927;
                    w = 0.023;
                    h = 0.035;
                    hauteur = 38;
                    largeur = 23;
                    break;
                case 6:
                    x = 0.048;
                    y = 0.927;
                    w = 0.023;
                    h = 0.036;
                    hauteur = 38;
                    largeur = 24;
                    break;
                case 7:
                    x = 0.025;
                    y = 0.927;
                    w = 0.023;
                    h = 0.035;
                    hauteur = 37;
                    largeur = 24;
                    break;
                default:
                    x = 0;
                    y = 0.927;
                    w = 0.024;
                    h = 0.034;
                    hauteur = 36;
                    largeur = 24;
                    break;
            }
            break;
        case SAUTE_DROITE:
            x = 0;
            y = 0.821;
            w = 0.023;
            h = 0.037;
            hauteur = 38;
            largeur = 24;
            break;
        case DESCEND_DROITE:
            x = 0.024;
            y = 0.821;
            w = 0.022;
            h = 0.032;
            hauteur = 34;
            largeur = 23;
            break;
        case COLLISION_SOL_DROITE:
            x = 0.046;
            y = 0.821;
            w = 0.020;
            h = 0.026;
            hauteur = 27;
            largeur = 21;
           break;
        case SAUTE_GAUCHE:
            x = 0;
            y = 0.784;
            w = 0.023;
            h = 0.036;
            hauteur = 38;
            largeur = 24;
            break;
        case DESCEND_GAUCHE:
            x = 0.024;
            y = 0.784;
            w = 0.022;
            h = 0.032;
            hauteur = 34;
            largeur = 23;
            break;
        case COLLISION_SOL_GAUCHE:
            x = 0.046;
            y = 0.784;
            w = 0.020;
            h = 0.026;
            hauteur = 27;
            largeur = 21;
            break;
        default:
            break;
    }
}

void Graphique::afficherVies(int nombreVies, int degats)
{
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

    // Cadre blanc
    glBegin(GL_TRIANGLE_STRIP);
        glColor4ub(255,255,255,0);
        glVertex2d(40,40);
        glColor4ub(255,255,255,80);
        glVertex2d(40,120);
        glColor4ub(255,255,255,80);
        glVertex2d(145,40);
        glColor4ub(255,255,255,0);
        glVertex2d(145,120);
    glEnd();
    glColor3ub(255,255,255);

    glEnable(GL_TEXTURE_2D);

        afficherDegats(degats);
        for(int i = 0; i < nombreVies; i++)
        {
            int ecart = i * 20;
            glBindTexture(GL_TEXTURE_2D, m_IMG_joueur);
            glBegin(GL_TRIANGLE_STRIP);
                glTexCoord2f(    0,     0);            glVertex2d(45 + ecart, 100);
                glTexCoord2f(    0, 0 + 0.017);        glVertex2d(45 + ecart, 100 + 17);
                glTexCoord2f(0 + 0.015,     0);        glVertex2d(45 + 15 + ecart, 100);
                glTexCoord2f(0 + 0.015, 0 + 0.017);    glVertex2d(45 + 15 + ecart, 100 + 17);
            glEnd();
            glColor3ub(255,255,255);
        }

    glDisable(GL_BLEND);
    glDisable(GL_TEXTURE_2D);
}

void Graphique::afficherDegats(int degats)
{
    double x = 0;
    double y = 0;
    double w = 0;
    double h = 0;
    int largeur = 0;
    int hauteur = 0;
    int nombreChiffre = 1;
    int chiffre = 0;
    int couleur = 0;
    if(degats < 10)
        nombreChiffre = 1;
    else if(degats >= 10 && degats < 100)
        nombreChiffre = 2;
    else
        nombreChiffre = 3;



    for(int i = 1; i <= nombreChiffre; i++)
    {
        int chiffreTemp = 0;
        if(i == 3)
        {
            if(nombreChiffre == 3)
                chiffre = degats / 100;
        }
        if(i == 2)
        {
            if(nombreChiffre == 3)
            {
                chiffreTemp = (degats / 100);
                chiffreTemp = chiffreTemp * 100;
                chiffreTemp = degats - chiffreTemp;
                chiffre     = (degats / 100);
                chiffre     = chiffre * 10;
                chiffre     = (degats / 10) - chiffre;
            }
            if(nombreChiffre == 2)
            {
                chiffre     = (degats / 10);
            }
        }
        if(i == 1)
        {
            if(nombreChiffre == 3)
            {
                chiffreTemp = (degats / 100);
                chiffreTemp = chiffreTemp * 100;
                chiffreTemp = degats - chiffreTemp;
                chiffre     = (degats / 100);
                chiffre     = chiffre * 10;
                chiffre     = (degats / 10) - chiffre;
                chiffre     = chiffre * 10;
                chiffreTemp = chiffreTemp - chiffre;
                chiffre     = chiffreTemp;
            }
            if(nombreChiffre == 2)
            {
                chiffreTemp = (degats / 10);
                chiffreTemp = chiffreTemp * 10;
                chiffre      = degats - chiffreTemp;
            }
            if(nombreChiffre == 1)
            {
                chiffre      = degats;
            }
        }

        getInfoDegats(chiffre, x, y, w, h, largeur, hauteur);


        int ecart = i * (largeur + 2);
        if(chiffre == 3)
            ecart = i * (largeur + 3);
        if(chiffre == 2)
            ecart = i * (largeur + 1);
        if(chiffre == 1)
            ecart = i * (largeur + 5);

        glBindTexture(GL_TEXTURE_2D, m_IMG_degats);
        glBegin(GL_TRIANGLE_STRIP);
            couleur  = degats;

            if(degats < 80)
            {
                couleur = 255;
                glColor3ub(couleur,255,255);
            }
            else
            {
                couleur = 255;
                glColor3ub(couleur,0,0);
            }
            glTexCoord2f(    x,     y);            glVertex2d(115 - ecart , 60);
            glTexCoord2f(    x, y + h);        glVertex2d(115 - ecart, 60 + hauteur);
            glTexCoord2f(x + w,     y);        glVertex2d(115 - ecart + largeur, 60);
            glTexCoord2f(x + w, y + h);    glVertex2d(115 - ecart + largeur, 60 + hauteur);
        glEnd();
        glColor3ub(255,255,255);
    }

    glBindTexture(GL_TEXTURE_2D, m_IMG_degats);
    glBegin(GL_TRIANGLE_STRIP);
        glTexCoord2f(       0.6,       0.5);            glVertex2d(115 , 60);
        glTexCoord2f(       0.6, 0.5 + 0.3);        glVertex2d(115 , 60 + 15);
        glTexCoord2f(0.6 + 0.045,       0.5);        glVertex2d(115 + 18, 60);
        glTexCoord2f(0.6 + 0.045, 0.5 + 0.3);    glVertex2d(115 + 18, 60 + 15);
    glEnd();
    glColor3ub(255,255,255);
}

void Graphique::getInfoDegats(int chiffre, double &x, double &y, double &w, double &h, int &largeur, int &hauteur)
{
    switch(chiffre)
    {
        case 0:
            x = 0;
            y = 0.5;
            w = 0.04;
            h = 0.5;
            hauteur = 27;
            largeur = 18;
            break;
        case 1:
            x = 0.065;
            y = 0.5;
            w = 0.035;
            h = 0.47;
            hauteur = 26;
            largeur = 13;
            break;
        case 2:
            x = 0.120;
            y = 0.5;
            w = 0.04;
            h = 0.5;
            hauteur = 27;
            largeur = 18;
            break;
        case 3:
            x = 0.181;
            y = 0.5;
            w = 0.038;
            h = 0.5;
            hauteur = 27;
            largeur = 17;
            break;
        case 4:
            x = 0.240;
            y = 0.5;
            w = 0.042;
            h = 0.48;
            hauteur = 26;
            largeur = 19;
            break;
        case 5:
            x = 0.305;
            y = 0.5;
            w = 0.038;
            h = 0.48;
            hauteur = 26;
            largeur = 16;
            break;
       case 6:
            x = 0.363;
            y = 0.5;
            w = 0.04;
            h = 0.5;
            hauteur = 27;
            largeur = 18;
            break;
       case 7:
            x = 0.424;
            y = 0.5;
            w = 0.04;
            h = 0.48;
            hauteur = 26;
            largeur = 18;
            break;
       case 8:
            x = 0.483;
            y = 0.5;
            w = 0.04;
            h = 0.5;
            hauteur = 27;
            largeur = 18;
            break;
       case 9:
            x = 0.543;
            y = 0.5;
            w = 0.04;
            h = 0.5;
            hauteur = 27;
            largeur = 18;
            break;
        default:
            break;
    }
}

void Graphique::afficherSolRessucite(float joueurX, float joueurY)
{
    glBegin(GL_TRIANGLE_STRIP);
        glColor4ub(255,0,0,255);
        glVertex2d(joueurX - 20,joueurY - 4);
        glColor4ub(0,255,0,255);
        glVertex2d(joueurX - 20,joueurY);
        glColor4ub(0,0,255,255);
        glVertex2d(joueurX + 40,joueurY - 4);
        glColor4ub(255,0,255,255);
        glVertex2d(joueurX + 40,joueurY);
    glEnd();
    glColor3ub(255,255,255);
}
