#include "Jeu.h"
#include "Moteur.h"

//Moteur du jeu
extern Moteur moteur;

Jeu::Jeu()
{
    // Le joueur
    joueur = new Joueur();

    // Le temps
    m_tempsPrecedent = 0;
    m_tempsEcoule    = 0;

    // Partie commencer ?
    m_commencer = false;

    // Niveau et zone par default.
    m_niveau = 1;
    m_zone   = 1;

    // Par dfault, on affiche seulement 50 particules (Quand le joueur meurt par exemple).
    particules.resize(50);
}

Jeu::~Jeu()
{
    delete joueur;
}

bool Jeu::initialisation()
{
    // Charge la map
    charger(m_niveau, m_zone);

    Texte::creerTexte("Dmo technique par Orhan Donovan", 20);

    return Graphique::chargerTextures() && Audio::chargerSons();
}

/***************************
*
*         Affichage
*
*************************** */
void Jeu::afficher()
{
    Graphique::dessinerBackground();
    Graphique::dessinerDecors(objets, joueur->getPositionX(), joueur->getPositionY());
    Graphique::dessinerParticules(particules, m_tempsEcoule);
    Texte::afficherTexte();
    joueur->afficher();
    m_commencer = true;
}

/***************************
*
*         Gestion
*
*************************** */
void Jeu::gestion()
{
    // Le temps est important pour gerer le joueur
    m_tempsEcoule = SDL_GetTicks() - m_tempsPrecedent;

    // Si le bulletTime est activ.
    if(moteur.getEtatSpecial() == true)
        m_tempsEcoule = m_tempsEcoule / 3;

    // Gestion du joueur.
    joueur->gestion(objets, particules, m_tempsEcoule);

    // Silapartie estcommencer on peut gerer les objets.
    if(m_commencer)
    {
        Physique::gestionDecors(objets, joueur->getPositionX(), joueur->getPositionY(), m_tempsEcoule);
        IA::gestionIA(objets, joueur->getPositionX(), joueur->getPositionY(), m_tempsEcoule);
    }

    m_tempsPrecedent  = SDL_GetTicks();
}


/***************************
*
*         Niveau
*
*************************** */
void Jeu::charger(int niveau, int zone)
{
    m_commencer = false;
    int nombreObjet  = 0;
    int fondRecupere = 1;
    Objet objetRecupere;

    // Repertoire ou sont les niveaux.
    std::string fichierACharger = "../media/levels/";

    // Conversion de int a texte.
    std::ostringstream oss;
    std::ostringstream oss2;
    oss  << niveau;
    oss2 << zone;
    std::string niveauTexte = oss.str();
    std::string zoneTexte   = oss2.str();

    fichierACharger += zoneTexte + "-" + niveauTexte + ".lvl";

    std::ifstream fichier;
    fichier.open(fichierACharger.c_str(),  std::ios::in);

    // On recupere le nombre d'objet qu'il y aura dans le niveau
    fichier >> nombreObjet;

    for(int i = 0; i < nombreObjet;i++)
    {
        fichier >> objetRecupere.m_type >> objetRecupere.m_positionX >> objetRecupere.m_positionY;
        // Si l'on trouve le dpart, on y met le joueur
        if(objetRecupere.m_type == 506)
            joueur->initialiser(objetRecupere.m_positionX, objetRecupere.m_positionY);

        objets.push_back(objetRecupere);
    }

    // Rcupration du background
    fichier >>  fondRecupere;

    // Detruit le fond actuel puis met le nouveau
    Graphique::changerBackground(fondRecupere);

    fichier.close();
    // Supprime l'objet temporairement cre.
    objetRecupere.~Objet();
}

/***************************
*
*         Clavier
*
*************************** */
void Jeu::clavier(unsigned char k)
{
    joueur->clavier(k);

    if(k == SDLK_ESCAPE)
        moteur.demanderFin();
}

void Jeu::clavierRelacher(unsigned char k)
{
    joueur->clavierRelacher(k);
}

/***************************
*
*         Autres
*
*************************** */
void Jeu::changerNiveau(int niveauDemander)
{
    switch(niveauDemander)
    {
        case 4:
            if(m_zone != 7 || m_zone != 8 || m_zone != 9)
                m_zone += 3;
            else
                m_niveau += 3;
            break;
        case 3:
            if(m_zone != 1 || m_zone != 2 || m_zone != 3)
                m_zone -= 3;
            else
                m_niveau -= 3;
            break;
        case 2:
            if(m_zone != 3 && m_zone != 6 && m_zone != 9)
                m_zone += 1;
            else
                m_niveau += 1;
            break;
        case 1:
            if(m_zone != 1 && m_zone != 4 && m_zone != 7)
                m_zone -= 1;
            else
                m_niveau -= 1;
            break;
        default:
            break;
    }

    objets.clear();
    charger(m_zone, m_niveau);
}
