#include "Joueur.h"
#include "Moteur.h"

//Moteur du jeu
extern Moteur moteur;


Joueur::Joueur()
{
    m_joueurX     = 0;
    m_joueurY     = 0;
    m_vx          = 0;
    m_vy          = 0;
    m_nombreVies  = 5;
    m_degats      = 0;
    m_nombreSauts = 0;
    m_action      = ATTEND_DROITE;

    // Le joueur ne porte rien.
    m_demanderPorter = false;
    m_objetPorter    = AUCUN_OBJET;
}

bool Joueur::initialiser(float x, float y)
{
    // On met le joueur au dpart du niveau, et sa vitesse.
    m_joueurX     = x;
    m_joueurY     = y;
    m_vx          = 0;
    m_vy          = 0;
    m_nombreSauts = 0;
    m_action      = ATTEND_DROITE;

    // Le joueur ne porte plus rien.
    m_demanderPorter  = false;
    m_objetPorter     = AUCUN_OBJET;

    return true;
}


/***************************
*
*        Affichage
*
*************************** */
void Joueur::afficher()
{
    Graphique::dessinerJoueur(m_action, m_etat, m_direction, m_joueurX, m_joueurY, m_vx, m_vy);
    Graphique::afficherVies(m_nombreVies, m_degats);
}

/***************************
*
*        Gestion
*
*************************** */
void Joueur::gestion(std::vector<Objet> &objets, std::vector<Particule> &particules, int tempsEcoule)
{
    // Savoir avec quoi le joueur rentre en collision sur X et Y.
    int objetEnCollisionX = 0;
    int objetEnCollisionY = 0;

    // Prendre un objet.
    if(m_demanderPorter == true && m_objetPorter == AUCUN_OBJET)
       m_demanderPorter = prendreObjet(objets);

    // Tenir un objet.
    if(m_objetPorter != AUCUN_OBJET && m_demanderPorter == true)
        tenirObjet(objets);

    // Jeter un objet
    if(m_objetPorter != AUCUN_OBJET && m_demanderPorter == false)
        jeterObjet(objets);

    // Saute dans le vide
    if(m_joueurY < - 10)
    {
        m_etat = RESSUCITE;
        jeterObjet(objets);
        m_degats      = 0;
        Graphique::initParticules(particules, m_joueurX, m_joueurY);
    }

    // Gestion de la physique ( collisions, vitesse, ... ) du joueur.
    Physique::mouvementsJoueur(m_action, m_direction, m_demanderPorter, m_vx, m_vy, m_nombreSauts, droite, gauche, tempsEcoule);
    Physique::gestionJoueur(objets, m_joueurX, m_joueurY, m_vx, m_vy, objetEnCollisionX, objetEnCollisionY, m_lastJoueurX, m_lastJoueurY, m_etat, tempsEcoule);
    Physique::antibugJoueur(objets, m_joueurX, m_joueurY, m_vy, m_vx, m_etat);
    gestionObjetJoueur(objets, objetEnCollisionX, objetEnCollisionY);

    if(m_etat == RESSUCITE)
        Script::ressuciter(m_etat, m_action, m_joueurX, m_joueurY, m_lastJoueurX, m_lastJoueurY, m_vx, m_vy, m_nombreVies, tempsEcoule);
}

void Joueur::gestionObjetJoueur(std::vector<Objet> &objets, int objetEnCollisionX, int objetEnCollisionY)
{
    int nombreObjets = objets.size();
    if(objetEnCollisionX > nombreObjets)
        objetEnCollisionX = 0;
    if(objetEnCollisionY > nombreObjets)
        objetEnCollisionY = 0;

    switch(objets[objetEnCollisionX].m_type)
    {
        // Les pieces.
        case 505:
            objets[objetEnCollisionX].m_etat = false;
            Audio::jouerPiecePrise();
            break;
        // Les cases pour changer de niveau.
        case 507:
            moteur.demanderChangementNiveau(4);
            objetEnCollisionY = 0;
            break;
        case 508:
            moteur.demanderChangementNiveau(3);
            objetEnCollisionY = 0;
            break;
        case 509:
            moteur.demanderChangementNiveau(1);
            objetEnCollisionY = 0;
            break;
        case 510:
            moteur.demanderChangementNiveau(2);
            objetEnCollisionY = 0;
            break;
        case 800:
            m_degats += 5;
            // Degats extremes !
            if(m_degats >= 999)
                m_degats = 999;

            if(m_direction == DROITE)
                m_vx -= m_degats / 50;
            if(m_direction == GAUCHE)
                m_vx += m_degats / 50;
            break;
        default:
            break;
    }

    switch(objets[objetEnCollisionY].m_type)
    {
        // Tonneau rebond.
        case 502:
            m_vy = -HAUTEUR_REBOND_TONNEAU;
            break;
        // Les pieces.
        case 505:
            objets[objetEnCollisionY].m_etat = false;
            Audio::jouerPiecePrise();
            break;
        // Les cases pour changer de niveau.
        case 507:
            moteur.demanderChangementNiveau(4);
            break;
        case 508:
            moteur.demanderChangementNiveau(3);
            break;
        case 510:
            moteur.demanderChangementNiveau(2);
            break;
        case 509:
            moteur.demanderChangementNiveau(1);
            break;
        case 800:
            //m_vy = -0.2;
            //objets[objetEnCollisionY].m_etat = false;
            break;
        default:
            break;
    }


}

/***************************
*
*         Clavier
*
*************************** */
void Joueur::clavier(unsigned char k)
{
    switch(k)
	{
	    case SDLK_w:
            if(haut == 0 && m_demanderPorter == false)
            {
                m_nombreSauts++;
                if(m_nombreSauts < 3)
                {
                    if(m_nombreSauts == 2)
                        m_vy = -HAUTEUR_SAUT_2_MARIO;
                    else
                        m_vy = -HAUTEUR_SAUT_1_MARIO;
                    haut = 1;
                }
            }
            break;
        case SDLK_d:
            droite = 1;
            break;
        case SDLK_a:
            gauche = 1;
            break;
        case SDLK_f:
            moteur.changerEtatBulletTime();
            break;
        case SDLK_g:
            m_demanderPorter = !m_demanderPorter;
            break;
		default:
			break;
	}
}

void Joueur::clavierRelacher(unsigned char k)
{
    switch(k)
	{
	    case SDLK_w:
            haut = 0;
            break;
        case SDLK_d:
            droite = 0;
            break;
        case SDLK_a:
            gauche = 0;
            break;
		default:
			break;
	}
}

/***************************
*
*   Intraction avec objet.
*
*************************** */
bool Joueur::prendreObjet(std::vector<Objet> objets)
{
    // On compte le nombre d'objet
    int nombreObjets  = objets.size();

    // Utilis pour recuperer les infos sur l'objet actuel et sur celui qu'on vas toucher
    int     l  = 24;
    double  x2 =  0;
    double  y2 =  0;
    int     l2 = 16;
    int     h2 = 16;

    for(int i = 0; i < nombreObjets; i++)
    {
        if(objets[i].m_type == 500 || objets[i].m_type == 501)
        {
            // Je rcupre les infos de l'objet qui est proche
            objets[i].getInfos(l2, h2);
            x2 = objets[i].m_positionX;
            y2 = objets[i].m_positionY;

            if(y2 == m_joueurY)
            {
                if(m_joueurX + l >= x2 && m_joueurX + l <= x2 + l2 && m_joueurX >= x2 - 10
                || m_joueurX >= x2 && m_joueurX <= x2 + l2 && m_joueurX + l <= x2 + l2 + 10)
                {
                   m_objetPorter = i;
                   Audio::jouerCaissePrise();
                   return true;
                }
            }
        }
    }
    return false;
}

void Joueur::jeterObjet(std::vector<Objet> &objets)
{
    if(m_direction == GAUCHE)
        objets[m_objetPorter].m_vx     -= 0.2;
    else
        objets[m_objetPorter].m_vx     = 0.2;

    objets[m_objetPorter].m_vy     = -0.2;
    objets[m_objetPorter].m_action = JETER;
    m_objetPorter                  = AUCUN_OBJET;
}

void Joueur::tenirObjet(std::vector<Objet> &objets)
{
    objets[m_objetPorter].m_vx        = 0;
    objets[m_objetPorter].m_vy        = 0;
    objets[m_objetPorter].m_positionX = m_joueurX - 4;
    objets[m_objetPorter].m_positionY = m_joueurY + 34 + 5;
    objets[m_objetPorter].m_action    = PORTER;
}

/***************************
*
*       Rcupration
*
*************************** */
float Joueur::getPositionX()
{
    return m_joueurX;
}

float Joueur::getPositionY()
{
    return m_joueurY;
}
