#include "Physique.h"
#include "../Moteur.h"

//Moteur du jeu
extern Moteur moteur;


/***************************
*
*       Partie joueur
*
*************************** */
void Physique::gestionJoueur(std::vector<Objet> &objets, float &m_joueurX, float &m_joueurY, float &m_vx, float &m_vy, int &objetEnCollisionX, int &objetEnCollisionY, float &lastJoueurX, float &lastJoueurY, int etat, int tempsEcoule)
{
    // On test si le joueur peut bouger
    bool bougerY2 = false;
    bool bougerX2 = false;

    if(etat != RESSUCITE)
    {
        bougerY2 = joueurBougerY(objets, m_joueurX, m_joueurY, m_vx, m_vy, objetEnCollisionX, objetEnCollisionY, lastJoueurX, lastJoueurY, tempsEcoule);
        bougerX2 = joueurBougerX(objets, m_joueurX, m_joueurY, m_vx, m_vy, objetEnCollisionX, objetEnCollisionY, tempsEcoule);

        if(bougerY2)
        {
            m_vy += GRAVITE * tempsEcoule;
            // On limite la vitesse maximum et minimum des caisses
            if (m_vy > VITESSE_MAX_JOUEUR)
                m_vy = VITESSE_MAX_JOUEUR;
            if (m_vy < -VITESSE_MAX_JOUEUR)
                m_vy = -VITESSE_MAX_JOUEUR;
        }

        if(bougerX2)
        {
            // On limite la vitesse maximum et minimum du joueur
            if (m_vx > VITESSE_MAX_JOUEUR)
                m_vx = VITESSE_MAX_JOUEUR;
            if (m_vx < -VITESSE_MAX_JOUEUR)
                m_vx = -VITESSE_MAX_JOUEUR;
        }
    }

    // On effectue les changements.
    m_joueurY -= m_vy * tempsEcoule;
    m_joueurX += m_vx * tempsEcoule;
}


bool Physique::joueurBougerY(std::vector<Objet> &objets, float &joueurX, float &joueurY, float &m_vx, float &m_vy, int &objetEnCollisionX, int &objetEnCollisionY, float &lastJoueurX, float &lastJoueurY, int tempsEcoule)
{
    // On recompte les objets
    int nombreObjets  = objets.size();

    // On recupere des infos sur l'objet actuel (dimensions, positions, ...)
    double  x = joueurX;
    double  y = joueurY;
    int     l = 24;
    int     h = 34;

    // Pour recuperer des infos sur l'objet qui vas etre en collision.
    double  x2 = 0;
    double  y2 = 0;
    int     l2 = 32;
    int     h2 = 32;

    // Pour chaque objet existant
    for(int i = 0; i < nombreObjets; i++)
    {
        /* ---------------------------------- */
        /*                                    */
        /* Collisions avec le bas du joueur   */
        /*                                    */
        /* ---------------------------------- */
        // Si l'objet se trouve pres de notre personnage ...
        if(objets[i].m_positionX >= x - 90 && objets[i].m_positionX + l2 <= x + l + 90 && objets[i].m_positionY <= y + h + 40  && objets[i].m_positionY >= y - 60
            && objets[i].m_etat == true   // ... Que l'objet qu'on veut toucher est toujours la.
            && NO_COLLISION()
            )
        {
            // On rcupre les dimensions et la position de l'objet proche.
            objets[i].getInfos(l2, h2);
            x2 = objets[i].m_positionX;
            y2 = objets[i].m_positionY;


            // Si un des coin de la caisse proche touche notre objet
            if(x     > x2 && x     <= x2 + l2  // Coin gauche de l'objet.
            || x + l > x2 && x + l <= x2 + l2  // Coin droit  de l'objet.
            || x     < x2 && x + l >= x2 + l2  // Notre objet est plus large.
            || x     > x2 && x + l <= x2 + l2) // Notre objet est plus petit.
            {

                if(y2 + h2 <= y + 20 && m_vy >= 0)
                {
                    // Et si il touche ou vas toucher le haut de l'objet
                    if(y - (m_vy * tempsEcoule) - 1  <= y2 + h2 - ((objets[i].m_vy * tempsEcoule) / 3))
                    {
                        objetEnCollisionY = i;
                        if(TRAVERSABLE_JOUEUR())
                        {
                            // Si c'est un tonneau qui fait sauter on vole !
                            // Si la vitesse de chute de notre objet est suprieur  1 au moment de la collision ...
                            if(m_vy > 1)
                                m_vy = -0.03; // On fait rebondir le joueur.
                            else
                                m_vy = 0; // Sinon le joueur est atteint par la vitesse de l'objet de dessous.

                            // Derniere endroit ou le joueur touchais le sol.
                            lastJoueurX = joueurX;
                            lastJoueurY = joueurY;

                            // Le joueur est atteint par la vitesse de l'objet ( Exemple : Le sol bouge, le joueur bouge avec )
                            // if(objets[i].m_vx != 0)
                            //   m_vx = objets[i].m_vx - 0.1;

                            // On aligne le joueur sur l'objet qui est sous lui ( Pour eviter que le joueur vole un peu au dessus ).
                            if(y2 + h2 - 2 <= y + 2 || objets[i].m_type == 110) // On l'aligne seulement si l'objet n'est pas trop haut par rapport au joueur.
                                joueurY = y2 + h2;

                            return false;
                        }
                    }
                }


                /* ---------------------------------- */
                /*                                    */
                /* Collisions avec le haut du joueur */
                /*                                    */
                /* ---------------------------------- */
                if(y2 + h2 + 5 >= y + h - 2 && m_vy < 0)
                {
                    // Et si il touche ou vas toucher le haut de l'objet + sa futur position + une marge
                    if(y + h -(m_vy * tempsEcoule) >= y2 && y <= y2)
                    {
                        if(objets[i].m_type != 502)
                            objetEnCollisionY = i;
                        if(TRAVERSABLE_JOUEUR() && objets[i].m_type != 110)
                        {
                            if(m_vy < -1)
                                m_vy = 0.1; // On fait rebondir le joueur.
                            else
                                m_vy = 0.01; // Sinon le joueur est atteint par la vitesse de l'objet de dessus.

                            // Afin d'eviter des bugs du  la vitesse, je dcalle le joueur avant de l'aligner sur l'objet de dessous.
                            if(m_vy < -0.3)
                                joueurY = y2;

                            return false;
                        }
                    }
                }
            }
        }
    }
    return true;
}

bool Physique::joueurBougerX(std::vector<Objet> &objets, float &m_joueurX, float &m_joueurY, float &m_vx, float &m_vy, int &objetEnCollisionX, int &objetEnCollisionY, int tempsEcoule)
{
    // On compte le nombre d'objet
    int nombreObjets  = objets.size();

    // Utilis pour recuperer les infos sur l'objet actuel et sur celui qu'on vas toucher
    double  x = m_joueurX;
    double  y = m_joueurY;
    int     l = 24;
    int     h = 34;
    double  x2 = 0;
    double  y2 = 0;
    int     l2 = 16;
    int     h2 = 16;

    for(int i = 0; i < nombreObjets; i++)
    {
        // Teste si l'objet est proche de celui actuel
        if(objets[i].m_positionX >= x - 90 && objets[i].m_positionX <= x + l + 90 && objets[i].m_positionY >= y - 40 && objets[i].m_positionY <= y + h + 40
            && objets[i].m_etat == true // L'objet qu'on vas tester est "en vie"
            && NO_COLLISION()
            )
        {

            // Je rcupre les infos de l'objet qui est proche
            objets[i].getInfos(l2, h2);
            x2 = objets[i].m_positionX;
            y2 = objets[i].m_positionY;

            if(y     >= y2  && y     <  y2 + h2 // Partie Basse
            || y + h >  y2  && y + h <= y2 + h2 // Partie Haute
            || y     <  y2  && y + h >= y2 + h2 // Notre objet est plus grand.
            || y     >  y2  && y + h <= y2 + h2 // Notre objet est plus petit.
            )
            {
                if(x2 > x - 10 && m_vx > 0)
                {
                        // Si l'objet qui est pas loin touche touche le coter droit
                        if(x + l + (m_vx * tempsEcoule) >= x2)
                        {
                            objetEnCollisionX = i;
                            if(TRAVERSABLE_JOUEUR() && objets[i].m_type != 110)
                            {
                                m_vx = 0;

                                if(m_vy > 0 )
                                    m_joueurX -= 2;

                                // Si l'ecart n'est pas enorme
                                if(m_joueurX + l - 2<= x2 + 2)
                                    m_joueurX = x2 - l - 1;

                                    return false;
                            }
                        }
                }

                if(x2 + l2 < x + 10 && m_vx < 0)
                {
                    // Si l'objet percute un objet de gauche
                    if(x + (m_vx * tempsEcoule) <= x2 + l2)
                    {
                        objetEnCollisionX = i;
                        if(TRAVERSABLE_JOUEUR() && objets[i].m_type != 110)
                        {
                            m_vx = 0;

                            if(m_vy > 0 )
                                m_joueurX += 2;

                            if(m_joueurX + 2 >= x2 + l2 - 2)
                                m_joueurX = x2 + l2 + 1;

                            return false;
                        }
                    }
                }
            }
        }
    }
    return true;
}

void Physique::antibugJoueur(std::vector<Objet> &objets, float &m_joueurX, float &m_joueurY, float m_vy, float m_vx, int etat)
{
    // On compte le nombre d'objet
    int nombreObjets  = objets.size();

    // Utilis pour recuperer les infos sur l'objet actuel et sur celui qu'on vas toucher
    double  x = m_joueurX;
    double  y = m_joueurY;
    int     l = 24;
    int     h = 34;
    double  x2 = 0;
    double  y2 = 0;
    int     l2 = 16;
    int     h2 = 16;

    for(int i = 0; i < nombreObjets; i++)
    {
        // Teste si l'objet est proche de celui actuel
        if(objets[i].m_positionX >= x - 40 && objets[i].m_positionX <= x + l + 40 && objets[i].m_positionY >= y - 20 && objets[i].m_positionY <= y + h + 20
            && objets[i].m_etat == true // L'objet qu'on vas tester est "en vie"
            && NO_COLLISION()
            && TRAVERSABLE_JOUEUR()
            && etat != RESSUCITE)
        {
            // Je rcupre les infos de l'objet qui est proche
            objets[i].getInfos(l2, h2);
            x2 = objets[i].m_positionX;
            y2 = objets[i].m_positionY;

            // Cas ou le joueur est enfoncer dans le sol.
            if(y2 + h2 - 1 > y && y2 + h2 < y + h)
            {
                // Si un des coin de la caisse proche touche notre objet
                if(x     > x2 && x     <= x2 + l2  // Coin gauche de l'objet.
                || x + l > x2 && x + l <= x2 + l2  // Coin droit  de l'objet.
                || x     < x2 && x + l >= x2 + l2  // Notre objet est plus large.
                || x     > x2 && x + l <= x2 + l2) // Notre objet est plus petit.
                {
                    m_joueurY += 5;
                }

            }
        }
    }
}

void Physique::mouvementsJoueur(int &action, int &direction, bool demanderPorter, float &vx, float &vy, int &nombreSauts, int droite, int gauche, int tempsEcoule)
{
    // On recule
    if(gauche == 1 && droite == 0)
    {
        if(vy == 0)
        {
            if(vx <= -0.3 && demanderPorter == false)
                vx = -0.3;
            else if(vx <= -0.15 && demanderPorter == true)
                vx = -0.15;
            else
                vx -= 0.05;
        }
        else
        {
            if(vx <= -0.3)
                vx = -0.3;
            else
                vx -= 0.02;
        }
        direction = GAUCHE;
    }
    // Si la touche droite est enfoncer on avance
    if(droite == 1 && gauche == 0)
    {
        if(vy == 0)
        {
            if(vx >= 0.3 && demanderPorter == false)
                vx = 0.3;
            else if(vx >= 0.15 && demanderPorter == true)
                vx = 0.15;
            else
                vx += 0.05;

        }
        else
        {
            if(vx >= 0.3)
                vx = 0.3;
            else
                vx += 0.02;
        }
        direction = DROITE;
    }

    if(vx > 0.1 && vy == 0)
        action = COURS_DROITE;

    if(vx < -0.1 && vy == 0)
        action = COURS_GAUCHE;

    if(vy < 0)
    {
        if(vx < -0.1)
            vx += 0.02;
        if(vx > 0.1)
            vx -= 0.02;
    }



    if(vy == 0)
    {
        // Si le joueur touche le sol on remet a zero lenombredesaut fait.
        nombreSauts = 0;

        // Avec la vitesse le joueur glisse.
        if(vx < -0.05 || vx > 0.05)
        {
            if(vx < -0.05 && gauche == 0)
                vx += GRAVITE * tempsEcoule * 1.3;
            if(vx > 0.05 && droite == 0)
                vx -= GRAVITE * tempsEcoule * 1.3;
        }
        else
            vx = 0;

            // Cas ou les touchent gauches et droite sont appuys simultanment.
        if(droite == 1 && gauche == 1)
        {
            if(vx < -0.05 || vx > 0.05)
            {
                if(vx < -0.05)
                    vx += GRAVITE * tempsEcoule * 1.3;
                if(vx > 0.05)
                    vx -= GRAVITE * tempsEcoule * 1.3;
            }
            else
                vx = 0;
        }
    }
}
