#include "Physique.h"
#include "../Moteur.h"

//Moteur du jeu
extern Moteur moteur;

Physique::Physique()
{

}

/***************************
*
*       Partie decors
*
*************************** */
void Physique::gestionDecors(std::vector<Objet> &objets, float positionJoueurX, float positionJoueurY, int tempsEcoule)
{

    // Pour chaque objet de la scene ...
    for(unsigned int i = 0; i < objets.size(); i++)
    {
        // Si c'est une caisse ou un tonneau.
        if(objets[i].m_type == 500 || objets[i].m_type == 501 || objets[i].m_type == 800 || objets[i].m_type == 801)
        {
            // Si celui-ci se trouve  l'cran ( Avec une marge ) et est en "vie".
            if (objets[i].m_positionX > positionJoueurX - (moteur.getLargeurEcran() / 2) - 20 && objets[i].m_positionX < positionJoueurX + moteur.getLargeurEcran() && objets[i].m_positionY > -20 && objets[i].m_positionY < positionJoueurY + moteur.getHauteurEcran() && objets[i].m_action != PORTER)
            {
                // On test si l'objet peut bouger
                bool bougerX = deplacementSurX(i, objets, positionJoueurX, positionJoueurY, tempsEcoule);
                bool bougerY = deplacementSurY(i, objets, positionJoueurX, positionJoueurY, tempsEcoule, bougerX);

                // Si la caisse peut descendre on la fait descendre.
                if(bougerY)
                {
                    objets[i].m_vy += GRAVITE * tempsEcoule;
                    // On limite la vitesse maximum et minimum des caisses
                    if (objets[i].m_vy > VITESSE_CAISSE)
                        objets[i].m_vy = VITESSE_CAISSE;
                    if (objets[i].m_vy < -VITESSE_CAISSE)
                        objets[i].m_vy = -VITESSE_CAISSE;
                }

                if(bougerX)
                {
                    // On limite la vitesse maximum et minimum des caisses
                    if (objets[i].m_vx > VITESSE_CAISSE)
                        objets[i].m_vx = VITESSE_CAISSE;
                    if (objets[i].m_vx < -VITESSE_CAISSE)
                        objets[i].m_vx = -VITESSE_CAISSE;
                }
                // On effectue les changements.
                objets[i].m_positionY -= objets[i].m_vy * tempsEcoule;
                objets[i].m_positionX += objets[i].m_vx * tempsEcoule;
            }
        }
    }
}

bool Physique::deplacementSurY(int id, std::vector<Objet> &objets, float positionJoueurX, float positionJoueurY,  int tempsEcoule, bool bougerX)
{
    // On recompte les objets
    int nombreObjets  = objets.size();

    // On recupere des infos sur l'objet actuel (dimensions, positions, ...)
    double  x = objets[id].m_positionX;
    double  y = objets[id].m_positionY;
    int     l = 16;
    int     h = 16;
    objets[id].getInfos(l, h);

    // Pour recuperer des infos sur l'objet qui vas etre en collision.
    double  x2 = 0;
    double  y2 = 0;
    int     l2 = 16;
    int     h2 = 16;

    // Pour chaque objet existant
    for(int i = 0; i < nombreObjets; i++)
    {
        // Teste si l'objet est proche de celui actuel
        if(objets[i].m_positionX >= x - 50 && objets[i].m_positionX <= x + l + 50 && objets[i].m_positionY >= y - 70 && objets[i].m_positionY <= y + h + 70
            && i != id
            && objets[i].m_etat == true // L'objet qu'on vas tester est "en vie"
            && TRAVERSABLE_OBJETS()
            && objets[i].m_action != PORTER)
        {

                // On rcupre les dimensions et la position de l'objet qui est peut etre en collision.
                objets[i].getInfos(l2, h2);
                x2 = objets[i].m_positionX;
                y2 = objets[i].m_positionY;


                // Si un des coin de la caisse proche touche notre objet
                if(x     > x2 && x     <= x2 + l2  // Coin gauche de l'objet.
                || x + l > x2 && x + l <= x2 + l2  // Coin droit  de l'objet.
                || x     < x2 && x + l >= x2 + l2  // Notre objet est plus large.
                || x     > x2 && x + l <= x2 + l2) // Notre objet est plus petit.
                {
                    /* ---------------------------------- */
                    /*                                    */
                    /* Collisions avec le bas de l'objet  */
                    /*                                    */
                    /* ---------------------------------- */
                    if(y2 + h2 - 20 <= y && objets[id].m_vy >= 0)
                    {
                        // Et si il touche ou vas toucher le haut de l'objet
                        if(y - (objets[id].m_vy * tempsEcoule) - 1 <= y2 + h2 + (objets[i].m_vy * tempsEcoule))
                        {
                            // Si c'est un monstre on rebondis sur lui et si l'objet actuel n'est pas un monstre !
                            if(objets[i].m_type > NOMBRE_MONSTRES && objets[id].m_type < NOMBRE_MONSTRES)
                            {
                                objets[id].m_vy  = -0.4;
                                objets[i].m_vx += 0.4;
                                objets[i].m_vy = -0.4;
                            }
                            // Si c'est un tonneau qui fait sauter on vole !
                            else if(objets[i].m_type == 502)
                                objets[id].m_vy = -HAUTEUR_REBOND_TONNEAU;
                            // Sinon on met sa vitesse a 0.
                            else
                            {
                                if(objets[id].m_vy > 0.5)
                                    objets[id].m_vy = -0.05; // On fait rebondir le joueur.
                                else
                                    objets[id].m_vy = objets[i].m_vy; // Sinon l'objet est atteint par la vitesse de l'objet dudessous

                                // Afin d'eviter des bugs du  la vitesse, je dcalle le joueur avant de l'aligner sur l'objet de dessous.
                                objets[id].m_positionY += 5;

                                // Et on reduit la vitesse de l'objet ( friction avec le sol )
                                if(objets[i].m_vx < -0.05 || objets[i].m_vx > 0.05)
                                    objets[id].m_vx = objets[i].m_vx;
                                else
                                    objets[id].m_vx -= objets[id].m_vx / 8;
                            }

                            // On aligne l'objet sur celui d'en dessous.
                            if(y2 + h2 - 2 <= objets[id].m_positionY + 2)
                                objets[id].m_positionY = y2 + h2;

                            return false;
                        }
                    }


                /* ---------------------------------- */
                /*                                    */
                /* Collisions avec le haut de l'objet */
                /*                                    */
                /* ---------------------------------- */
                if(y2 + h2 + 5 >= y + h - 2 && objets[id].m_vy < 0)
                {
                    // Et si il touche ou vas toucher le haut de l'objet + sa futur position + une marge
                    if(y + h -(objets[id].m_vy * tempsEcoule) >= y2 - (objets[i].m_vy * tempsEcoule))
                    {
                        // L'objet du dessus voit sa vitesse augmenter par celle de notre objet de 40%.
                        if(objets[i].m_type == 500)
                            objets[i].m_vy    = (objets[id].m_vy * 0.8);

                        // L'objet du dessous perd toute sa vitesse
                        objets[id].m_vy  = 0.1;

                        objets[id].m_positionY -= 5;

                        // On aligne l'objet sur celui de dessus.
                        if(y2 >= y + h - 4)
                            objets[id].m_positionY = y2 - h;

                        // Et on arrete le test des collisions avec les autres objets
                        return false;
                    }
                }
            }
        }
    }

    return true;
}


bool Physique::deplacementSurX(int id, std::vector<Objet> &objets, float positionJoueurX, float positionJoueurY, int tempsEcoule)
{
    // On compte le nombre d'objet
    int nombreObjets  = objets.size();

    // Utilis pour recuperer les infos sur l'objet actuel et sur celui qu'on vas toucher
    double  x = objets[id].m_positionX;
    double  y = objets[id].m_positionY;
    int     l = 16;
    int     h = 16;
    double  x2 = 0;
    double  y2 = 0;
    int     l2 = 16;
    int     h2 = 16;

    // Recupere les infos de l'objet qu'on test
    objets[id].getInfos(l, h);

    for(int i = 0; i < nombreObjets; i++)
    {
        // Teste si l'objet est proche de celui actuel
        if(objets[i].m_positionX >= x - 80 && objets[i].m_positionX <= x + l + 80 && objets[i].m_positionY >= y - 40 && objets[i].m_positionY <= y + h + 40
            && objets[i].m_etat == true // L'objet qu'on vas tester est "en vie"
            && i != id
            && TRAVERSABLE_OBJETS()
            && objets[i].m_action != PORTER)
        {
            // Je rcupre les infos de l'objet qui est proche
            objets[i].getInfos(l2, h2);
            x2 = objets[i].m_positionX;
            y2 = objets[i].m_positionY;

            // Anti-bugs, verifie quela caisse est pas traverser
            if(y2 + h2 - 1 > y && y2 + h2 < y + h)
            {
                // Si un des coin de la caisse proche touche notre objet
                if(x     > x2 && x     <= x2 + l2  // Coin gauche de l'objet.
                || x + l > x2 && x + l <= x2 + l2  // Coin droit  de l'objet.
                || x     < x2 && x + l >= x2 + l2  // Notre objet est plus large.
                || x     > x2 && x + l <= x2 + l2) // Notre objet est plus petit.
                {
                    objets[id].m_positionY += 5;
                }

            }


            if(y     >= y2  && y     <  y2 + h2 // Partie Basse
            || y + h >  y2  && y + h <= y2 + h2 // Partie Haute
            || y     <  y2  && y + h >= y2 + h2 // Notre objet est plus grand.
            || y     >  y2  && y + h <= y2 + h2 // Notre objet est plus petit.
            )
            {
                if(x + l < x2 && objets[id].m_vx > 0)
                {
                    // Si l'objet qui est pas loin touche touche le coter droit
                    if(x + l + (objets[id].m_vx * tempsEcoule) >= x2)
                    {
                        if(objets[i].m_type == 500 || objets[i].m_type == 501)
                            objets[i].m_vx = objets[id].m_vx;

                        if(objets[id].m_type == 800 || objets[id].m_type == 801)
                            objets[id].m_action = MARCHE_GAUCHE;

                        objets[id].m_vx     = 0; // Sinon notre objet est un peu repousser par l'autre


                        // Si l'ecart n'est pas enorme
                        if(x + l - 2<= x2 + 2)
                            objets[id].m_positionX = x2 - l - 1;

                        return false;
                    }
                }


                if(x > x2 + l2 && objets[id].m_vx < 0)
                {
                    // Si l'objet percute un objet de gauche
                    if(x + (objets[id].m_vx * tempsEcoule) <= x2 + l2)
                    {
                        if(objets[i].m_type == 500 || objets[i].m_type == 501)
                            objets[i].m_vx = objets[id].m_vx;

                        if(objets[id].m_type == 800 || objets[id].m_type == 801)
                            objets[id].m_action = MARCHE_DROITE;

                        objets[id].m_vx  = 0;

                        if(x + 2 >= x2 + l2 - 2)
                            objets[id].m_positionX = x2 + l2 + 1;

                        return false;
                    }
                }
            }
        }
    }

    return true;
}
