#include "main.h"

using namespace std;
Moteur moteur;

int main(int argc, char *argv[])
{
    // Variables
    SDL_Event event;

    // Permet de gerer le taux d'images par seconde.
    unsigned int tempsActuel = 0;
    int fps = 35; // Le nombre d'images/secondes souhaits.

    /***************************
    *    Paramtres utilisateur
    *  --------------------------
    * (Pleine ecran, resolution )
    *************************** */
    if(!chargementConfiguration())
    {
        fprintf(stderr, "Impossible de charger le fichier de configuration");
        atexit(0);
    }


    /***************************
    *    Ecran et bibliotheques
    *  --------------------------
    *      (SDL, Fmod, ... )
    *************************** */
    if(!initialisation())
    {
        fprintf(stderr, "Erreur lors de l'initialisation de l'affichage");
        atexit(0);
    }

    /***************************
    *  Initilisation du moteur
    *  --------------------------
    *   (Textures, niveau, ...)
    *************************** */
    if(!moteur.initialisation())
    {
        fprintf(stderr, "Erreur lors de l'initialisation du moteur");
        atexit(0);
    }

    // Rpetition des touches
    SDL_EnableKeyRepeat(100, 100);
    while(!moteur.fin())
    {
        while(SDL_PollEvent(&event))
        {
            switch (event.type)
            {
                case SDL_QUIT: // Ask the end.
                    moteur.demanderFin();
                    break;
                case SDL_KEYDOWN:
                    moteur.clavier(event.key.keysym.sym);
                    break;
                case SDL_KEYUP:
                    moteur.clavierRelacher(event.key.keysym.sym);
                    break;
                default:
                    break;
            }
        }



        /**************************
        *    Gestion de la scene
        **************************/
        if(SDL_GetTicks() > (tempsActuel + 1000 / fps) )
        {
            // On met a jour la variable tempsActuel.
            tempsActuel = SDL_GetTicks();

            // Demander au moteur de s'occuper de la scene.
            moteur.afficherScene();
        }
        else
            SDL_Delay(5);

    }
    return EXIT_SUCCESS;
}


bool initialisation()
{
    // Cration d'un boolen pour verifier l'tat de l'initialisation
    bool etat = true;

    // Initialisation de Devil, Freetype
    ilInit();
    InitFontEngine();

    // Fsound
    FSOUND_Init(44100, 32, 0);

    // Initilisation de la fentre.
    if(SDL_Init(SDL_INIT_VIDEO))
    {
        fprintf(stderr, "Erreur d'initialisation de la SDL");
        etat = false;
    }

    // Alignement de la fenetre  l'cran.
    #if !defined(NDEBUG) && !defined(_MSC_VER)
        freopen( "CON", "w", stdout );
        freopen( "CON", "w", stderr );
    #endif
    SDL_putenv("SDL_VIDEO_WINDOW_POS=center");

    // Nom de la fentre.
    SDL_WM_SetCaption("Smash 2D par Hybrix", NULL);

    // Initialisation de l'ecran
    if (moteur.getFullScreen() == true)
        SDL_SetVideoMode(moteur.getLargeurEcran(), moteur.getHauteurEcran(), 32, SDL_OPENGL | SDL_FULLSCREEN);
    else
        SDL_SetVideoMode(moteur.getLargeurEcran(), moteur.getHauteurEcran(), 32, SDL_OPENGL);

    // Mise en place du plan 2D
    glMatrixMode( GL_PROJECTION ); // Pour modifier la Matrice.
    glLoadIdentity();
    gluOrtho2D(  0,  moteur.getLargeurEcran(),  0,  moteur.getHauteurEcran());
    //glEnable(GL_DEPTH_TEST);

    return etat;
}

void suppressions()
{
    FSOUND_Close();
    ShutdownFontEngine();
    ilShutDown();
    SDL_Quit();
}

bool chargementConfiguration()
{
    bool etat      = true;

    // Valeurs par dfault.
    int largeur    = 1024;
    int hauteur    =  768;
    int fullscreen =    0;

    // Rcupration du fichier.
    std::ifstream fichier;
    fichier.open("../media/player/config.hyb",  std::ios::in);

    // Scurite.
    if(fichier.bad())
        etat = false;

    // On recupere a config enregistr.
    fichier >> largeur >> hauteur >> fullscreen;

    // Et on enregistre tout dans le moteur.
    moteur.changerLargeur(largeur);
    moteur.changerHauteur(hauteur);
    moteur.changerFullScreen(fullscreen);

    fichier.close();

    return etat;
}

